from flask_login import current_user
from flask_socketio import join_room, leave_room
from app import hashids, socketio
from app.decorators import socketio_login_required
from app.models import User


@socketio.on('SUBSCRIBE User')
@socketio_login_required
def subscribe(user_hashid: str) -> dict:
    if not isinstance(user_hashid, str):
        return {
            'code': 400,
            'name': 'Bad Request',
            'description': 'Invalid User ID.'
        }

    user_id = hashids.decode(user_hashid)

    if not isinstance(user_id, int):
        return {
            'code': 400,
            'name': 'Bad Request',
            'description': 'Invalid User ID.'
        }

    user = User.query.get(user_id)

    if user is None:
        return {
            'code': 404,
            'name': 'Not Found',
            'description': 'User not found.'
        }

    if not (
        user == current_user
        or current_user.is_administrator
    ):
        return {
            'code': 403,
            'name': 'Forbidden',
            'description': 'Not allowed to subscribe to this user.'
        }

    join_room(f'/users/{user.hashid}')

    return {'code': 204, 'name': 'No Content'}


@socketio.on('UNSUBSCRIBE User')
@socketio_login_required
def unsubscribe(user_hashid: str) -> dict:
    if not isinstance(user_hashid, str):
        return {
            'code': 400,
            'name': 'Bad Request',
            'description': 'Invalid User ID.'
        }

    user_id = hashids.decode(user_hashid)

    if not isinstance(user_id, int):
        return {
            'code': 400,
            'name': 'Bad Request',
            'description': 'Invalid User ID.'
        }

    user = User.query.get(user_id)

    if user is None:
        return {
            'code': 404,
            'name': 'Not Found',
            'description': 'User not found.'
        }

    if not (
        user == current_user
        or current_user.is_administrator
    ):
        return {
            'code': 403,
            'name': 'Forbidden',
            'description': 'Not allowed to unsubscribe from this user.'
        }

    leave_room(f'/users/{user.hashid}')

    return {'code': 204, 'name': 'No Content'}
